module DemoDropPerpendicular exposing (..)

import Browser
import Debug exposing (log)
import GeometryFunctions exposing (..)
import GeometryTypes exposing (..)
import Html exposing (Attribute, Html, button, div, input, text)
import Html.Attributes as Att
import Html.Events exposing (onClick, onInput, onMouseEnter, onMouseLeave)
import Html.Events.Extra.Mouse as Mouse
import Svg exposing (..)
import Svg.Attributes
import Svg.Events
import SvgAuxiliary exposing (..)


main =
    Browser.sandbox
        { init = init
        , view = view
        , update = update

        -- subscriptions is stuff we want to listen to, beyond the input elements of our web app
        -- this could, for example, be web sockets ...
        }


type alias Model =
    { mousePosition : Vector
    , pointConfig : PointConfig
    }


type PointConfig
    = NoPoint
    | SinglePoint Vector
    | Line DirectedLine


type Msg
    = MouseMoved Mouse.Event
    | MouseClicked Mouse.Event


pixelWidth =
    1000


pixelHeight =
    600


init : Model
init =
    { mousePosition = { x = -100, y = -100 }
    , pointConfig = NoPoint
    }


update : Msg -> Model -> Model
update msg model =
    case msg of
        MouseMoved event ->
            let
                ( x, y ) =
                    event.offsetPos
            in
            { model | mousePosition = { x = x, y = y } }

        MouseClicked event ->
            let
                ( x, y ) =
                    event.offsetPos

                newPointConfig : PointConfig
                newPointConfig =
                    case model.pointConfig of
                        NoPoint ->
                            SinglePoint { x = x, y = y }

                        SinglePoint firstPoint ->
                            Line { from = firstPoint, to = { x = x, y = y } }

                        Line line ->
                            Line { from = line.to, to = { x = x, y = y } }
            in
            { model | pointConfig = newPointConfig }


css path =
    Html.node "link" [ Att.rel "stylesheet", Att.href path ] []


view : Model -> Html Msg
view model =
    div
        [ Att.class "container-fluid" ]
        [ css "https://maxcdn.bootstrapcdn.com/bootstrap/3.4.1/css/bootstrap.min.css"
        , Html.p [] [ Html.text "source can be found under ", Html.code [] [ Html.text "/elm-codebase/svg/DemoDropPerpendicular.elm" ] ]
        , Html.h2 []
            [ Html.text "Drop a line. Then it tracks to point on the line closest to the mouse." ]
        , div
            []
            [ Svg.svg
                [ Att.style "border" "1pt black solid"
                , Att.style "margin" "20px"
                , Svg.Attributes.width (String.fromFloat pixelWidth)
                , Svg.Attributes.height (String.fromFloat pixelHeight)
                , Mouse.onMove MouseMoved
                , Mouse.onClick MouseClicked
                , Att.style "margin-top" "2vh"

                --, Svg.Attributes.width "90vw"
                --, Svg.Attributes.height "80vh"
                ]
                (listOfSvgObjects model)

            -- no svg objects right now
            -- listOfSvgObjects
            ]
        ]


listOfSvgObjects : Model -> List (Svg Msg)
listOfSvgObjects model =
    let
        makeDot : Vector -> String -> Svg Msg
        makeDot v color =
            SvgAuxiliary.createDot
                [ Svg.Attributes.stroke "black"
                , Svg.Attributes.fill color
                , Svg.Attributes.strokeWidth "1"
                ]
                ( v.x, v.y )
                8
    in
    case model.pointConfig of
        NoPoint ->
            [ makeDot model.mousePosition "cornsilk" ]

        SinglePoint p ->
            [ makeDot p "cornsilk"
            , makeDot model.mousePosition "azure"
            ]

        Line line ->
            let
                u : Vector
                u =
                    line.from

                v : Vector
                v =
                    line.to

                closestPoint : Vector
                closestPoint =
                    pointOnLineClosestTo line model.mousePosition
            in
            [ SvgAuxiliary.createLine [ Svg.Attributes.stroke "black" ] ( u.x, u.y ) ( v.x, v.y )
            , makeDot closestPoint "grey"
            , makeDot u "cornsilk"
            , makeDot v "azure"
            ]
