module DemoRotateVector exposing (..)

import Browser
import Debug exposing (log)
import DominiksSolution exposing (..)
import GeometryFunctions exposing (..)
import GeometryTypes exposing (..)
import Html exposing (Attribute, Html, button, div, input, text)
import Html.Attributes as Att
import Html.Events exposing (onClick, onInput, onMouseEnter, onMouseLeave)
import Html.Events.Extra.Mouse as Mouse
import Result exposing (Result)
import Svg exposing (..)
import Svg.Attributes
import Svg.Events
import SvgAuxiliary exposing (..)


main =
    Browser.sandbox
        { init = init
        , view = view
        , update = update

        -- subscriptions is stuff we want to listen to, beyond the input elements of our web app
        -- this could, for example, be web sockets ...
        }


type alias Model =
    { mousePosition : Vector
    , message : Result String String
    , angleString : String
    , angle : Float
    , mouseDown : Bool
    }


type Msg
    = MouseDown Mouse.Event
    | MouseUp Mouse.Event
    | MouseMoved Mouse.Event
    | AngleInputChanged String
    | AngleButtonClicked


pixelWidth =
    1000


pixelHeight =
    600


initialAngle =
    60


initialAngleString =
    String.fromInt initialAngle


origin : Vector
origin =
    { x = pixelWidth / 2, y = pixelHeight / 2 }


init : Model
init =
    { mousePosition =
        { x = origin.x + 100, y = origin.y + 100 }
    , angleString = initialAngleString
    , angle = initialAngle
    , message = Result.Ok ("angle is " ++ initialAngleString ++ " degrees")
    , mouseDown = False
    }


update : Msg -> Model -> Model
update msg model =
    case msg of
        MouseDown event ->
            let
                ( x, y ) =
                    event.offsetPos
            in
            { model
                | mouseDown = True
                , mousePosition = { x = x, y = y }
            }

        MouseUp event ->
            { model | mouseDown = False }

        MouseMoved event ->
            if not model.mouseDown then
                model

            else
                let
                    ( x, y ) =
                        event.offsetPos
                in
                { model
                    | mousePosition = { x = x, y = y }
                }

        AngleInputChanged s ->
            { model | angleString = s }

        AngleButtonClicked ->
            case String.toFloat model.angleString of
                Just newAngle ->
                    let
                        asfasdfjkl =
                            Debug.log "correct angle" model.angleString
                    in
                    { model
                        | angle = newAngle
                        , message = Result.Ok ("angle is " ++ model.angleString)
                    }

                Nothing ->
                    let
                        asfasdfjkl =
                            Debug.log "wrong angle" model.angleString
                    in
                    { model
                        | message = Result.Err "angle has to be float"
                    }


css path =
    Html.node "link" [ Att.rel "stylesheet", Att.href path ] []


view : Model -> Html Msg
view model =
    div
        [ Att.class "container-fluid" ]
        [ css "https://maxcdn.bootstrapcdn.com/bootstrap/3.4.1/css/bootstrap.min.css"
        , Html.p [] [ Html.text "source can be found under ", Html.code [] [ Html.text "/elm-codebase/pp-geometry/DemoRotateVector.elm" ] ]
        , Html.h2 []
            [ Html.text "Enter an angle and watch the vector being rotated" ]
        , div []
            [ Html.text "angle"
            , Html.input
                [ Att.value model.angleString
                , Html.Events.onInput AngleInputChanged
                ]
                []
            , Html.button [ Html.Events.onClick AngleButtonClicked ]
                [ Html.text "change angle" ]
            ]
        , div []
            [ case model.message of
                Result.Ok s ->
                    Html.p
                        [ Att.style "color" "darkgreen" ]
                        [ Html.text s ]

                Result.Err s ->
                    Html.p
                        [ Att.style "color" "red" ]
                        [ Html.text s ]
            ]
        , div
            []
            [ Svg.svg
                [ Att.style "border" "1pt black solid"
                , Att.style "margin" "20px"
                , Svg.Attributes.width (String.fromFloat pixelWidth)
                , Svg.Attributes.height (String.fromFloat pixelHeight)
                , Mouse.onDown MouseDown
                , Mouse.onUp MouseUp
                , Mouse.onMove MouseMoved
                , Att.style "margin-top" "2vh"

                --, Svg.Attributes.width "90vw"
                --, Svg.Attributes.height "80vh"
                ]
                (listOfSvgObjects model)

            -- no svg objects right now
            -- listOfSvgObjects
            ]
        ]


listOfSvgObjects : Model -> List (Svg Msg)
listOfSvgObjects model =
    let
        makeDot : Vector -> String -> Svg Msg
        makeDot v color =
            SvgAuxiliary.createDot
                [ Svg.Attributes.stroke "black"
                , Svg.Attributes.fill color
                , Svg.Attributes.strokeWidth "1"
                ]
                ( v.x, v.y )
                8

        makeLine : Vector -> Vector -> Svg Msg
        makeLine u v =
            SvgAuxiliary.createLine [ Svg.Attributes.stroke "black" ] ( u.x, u.y ) ( v.x, v.y )

        endPoint : Vector
        endPoint =
            rotateLine model.angle origin model.mousePosition
    in
    [ makeLine origin model.mousePosition
    , makeLine origin endPoint
    , makeDot origin "cornsilk"
    , makeDot model.mousePosition "azure"
    , makeDot endPoint "pink"
    ]
